import {CloudBackground} from "../../component/fluid/cloud.tsx";
import {useEffect, useState} from "react";
import {createUseStyles} from "react-jss";
import {useWebsocket} from "../../hook/websocket/u-ws.tsx";
import {Resp} from "../../interface/response.ts";
import {useRegister} from "./hook/register.tsx";

const useClass = createUseStyles({
    '@global': {
        '@keyframes emerge': {
            '0%': {
                transform: 'scale(0) translate(-50%, -50%)',
                opacity: 0
            },
            '80%': {
                transform: 'scale(1.1) translate(-50%, -50%)',
                opacity: 1
            },
            '100%': {
                transform: 'scale(1) translate(-50%, -50%)',
                opacity: 1
            }
        }
    },
    container: {
        margin: "0",
        height: "100vh",
        // background: "linear-gradient(45deg, #e6e9f0, #eef1f5)",
        overflow: "hidden",
        position: "relative",
    },
    title: {
        width: '100%',
        display: "flex",
        justifyContent: "center",
        color: '#1661ab',
    },
    bubble: {
        position: "absolute",
        width: "100px",
        height: "100px",
        borderRadius: "50%",
        display: "flex",
        justifyContent: "center",
        alignItems: "center",
        textAlign: "center",
        cursor: "pointer",
        fontFamily: "'Microsoft Yahei', sans-serif",
        fontSize: "14px",
        color: "rgba(255, 255, 255, 0.9)",
        textShadow: "1px 1px 3px rgba(0,0,0,0.3)",
        transition: "transform 0.3s ease",
        transform: 'translate(-50%, -50%)',
        animation: 'emerge 0.5s ease-out forwards,float 6s 0.5s ease-in-out infinite',
        background: "radial-gradient(circle at 30% 30%,rgba(255, 255, 255, 0.8) 10%,rgba(255, 255, 255, 0.3) 50%,transparent 100%)",
        border: "2px solid rgba(255, 255, 255, 0.5)",
        boxShadow: "inset 0 -5px 15px rgba(255,255,255,0.3),0 5px 15px rgba(0,0,0,0.1)",
    }
})


interface Bubble {
    id: string;
    name: string;
    x: number;
    y: number;
    color: string;
    radius: number; // 新增半径属性
    angle: number;  // 新增角度属性
}

interface WSMessage {
    body: Client;
    time: number;
    type: "register" | "enter" | "leave" | "offer" | "answer"
}

interface Client {
    client_type: 'desktop' | 'mobile' | 'tablet';
    app_type: 'web';
    ip: number;
    name: string;
    id: string;
    register_at: string;
    offer: RTCSessionDescription;
    candidate: RTCIceCandidateInit;
}

function setupDataChannel(ch: RTCDataChannel) {
    ch.onopen = () => console.log('通道已打开！');
    ch.onmessage = (e) => handleFileChunk(e.data);
    ch.onclose = () => console.log('通道关闭');
}

// 接收文件块
function handleFileChunk(chunk: any) {
    console.log("[D] rtc file chunk =", chunk)
}


export const LocalSharing: React.FC = () => {
    const classes = useClass();
    const {id, name, register } = useRegister()
    const [rtc, setRTC] = useState<RTCPeerConnection| null>(null);
    const [client, setClient] = useState<Client| null>(null);
    const [clients, setClients] = useState<Client[]>([]);
    const {connect, close} = useWebsocket({})

    // 生成随机颜色
    const generateColor = () => {
        const hue = Math.random() * 360;
        return `hsla(${hue}, 
            ${Math.random() * 30 + 40}%, 
            ${Math.random() * 10 + 75}%, 0.9)`;
    };

    // 防碰撞位置生成
    const generateBubbles = () => {
        if (!clients) return []

        const BUBBLE_SIZE = 100;
        const centerX = window.innerWidth / 2;
        const centerY = window.innerHeight / 2;

        const bubbles: Bubble[] = [];
        let currentRadius = 0;
        let angleStep = (2 * Math.PI) / 6; // 初始6个位置

        for (let index = 0; index < clients.length; index++) {
            let attempt = 0;
            let validPosition = false;

            if (clients[index].id == id) {
                continue
            }

            while (!validPosition && attempt < 100) {
                // 螺旋布局算法
                const angle = angleStep * (index + attempt);
                const radius = currentRadius + (attempt * BUBBLE_SIZE * 0.8);

                // 极坐标转笛卡尔坐标
                const x = centerX + radius * Math.cos(angle);
                const y = centerY + radius * Math.sin(angle);

                // 边界检测
                const inBounds = x >= 0 && x <= window.innerWidth - BUBBLE_SIZE &&
                    y >= 0 && y <= window.innerHeight - BUBBLE_SIZE;

                // 碰撞检测
                const collision = bubbles.some(pos => {
                    const distance = Math.sqrt(
                        Math.pow(pos.x - x, 2) +
                        Math.pow(pos.y - y, 2)
                    );
                    return distance < BUBBLE_SIZE * 1.5;
                });

                if (inBounds && !collision) {
                    bubbles.push({
                        id: clients[index].id,
                        name: clients[index].name,
                        x: x,
                        y: y,
                        color: generateColor(),
                    } as Bubble);

                    // 动态调整布局参数
                    currentRadius = Math.max(currentRadius, radius);
                    angleStep = (2 * Math.PI) / Math.max(6, bubbles.length * 0.7);
                    validPosition = true;
                }

                attempt++;
            }
        }

        console.log('[D] generated bubbles =', bubbles);
        return bubbles;
    };

    const updateClients = async () => {
        setTimeout(async () => {
            const res = await fetch(`/api/ulocal/clients`)
            const jes = await res.json() as Resp<Client[]>
            console.log('[D] update clients =', jes)
            setClients(jes.data)
        }, 500)
    }

    const handleWSEvent = async (e: MessageEvent) => {
        const msg = JSON.parse(e.data) as WSMessage
        console.log('[D] ws event msg =', msg)
        switch (msg.type) {
            case "register":
                const reg_resp = JSON.parse(e.data) as { body: Client }
                setClient(reg_resp.body)
                break
            case "enter":
                await updateClients()
                break
            case "leave":
                await updateClients()
                break
            case "offer":
                break
            case  "answer":
                break
        }
    }

// 气泡点击处理
    const handleBubbleClick = async (bubble: Bubble) => {
        const offer = await rtc?.createOffer()
        await rtc?.setLocalDescription(offer)
        const data = {
            id: bubble.id,
            offer: offer,
        }
        await fetch('/api/ulocal/offer', {
            method: 'POST',
            headers: {"Content-Type": "application/json"},
            body: JSON.stringify(data)
        })
    };

    useEffect(() => {
        const fn = async () => {
            const reg_data = await register();
            console.log(`[D] register id = ${reg_data?.id}`);
            connect(`/api/ulocal/ws?id=${reg_data?.id}`, {fn: handleWSEvent})
            await updateClients()
            const rtc = new RTCPeerConnection({iceServers: [{urls: "stun:stun.qq.com:3478"}]})
            rtc.onicecandidate = async (e) => {
                if(e.candidate) {
                    await fetch('/api/ulocal/candidate')
                }
            }
            setRTC(rtc)

            return () => {close();}
        };
        fn()
    }, [])

    return <div className={classes.container}>
        <CloudBackground/>
        <h1 className={classes.title}>{name}</h1>
        {clients && generateBubbles().map(bubble => {
            return client ? (
                <div
                    key={bubble.id}
                    className={classes.bubble}
                    style={{
                        left: bubble.x,
                        top: bubble.y,
                        backgroundColor: bubble.color,
                        animationDelay:
                            `${Math.random() * 0.5}s, 
                     ${0.5 + Math.random() * 2}s`
                    }}
                    onClick={() => handleBubbleClick(bubble)}
                >
                    {bubble.name}
                </div>
            ) : null;
        })}
    </div>
}